@@ -22,6 +22,11 @@ setMethod ("coltab<-" , "SpatRaster",
 			layer <- 1
 		}
 		layer <- layer[1]-1
+		
+		if (is.null(value)) {
+			x@ptr$removeColors(layer-1)
+			return(x)
+		}
 
 		if (inherits(value, "list")) {
 			value <- value[[1]]
