@@ -4,65 +4,82 @@
 }
 
 
-.sampleCells <- function(x, size, method, replace) {
-	lonlat <- is.lonlat(x, perhaps=TRUE, warn=TRUE)
+.sampleCells <- function(x, size, method, replace, na.rm=FALSE, ext=NULL) {
+	r <- rast(x)
+	lonlat <- is.lonlat(r, perhaps=TRUE, warn=TRUE)
+	if (!is.null(ext)) {
+		r <- crop(rast(r), ext)
+	}
 	if (method == "random") {
+		n <- size
+		if (na.rm) {
+			size <- min(ncell(r)*2, size*5)
+		}
 		if (lonlat) {
 			m <- ifelse(replace, 1.5, 1.25)
 			n <- m * size
-			r <- yFromRow(x, 1:nrow(x))
-			w <- abs(cos(pi*r/180))
-			rows <- sample.int(nrow(x), n, replace=TRUE, prob=w)
-			cols <- sample.int(ncol(x), n, replace=TRUE)
-			cells <- cellFromRowCol(x, rows, cols)
+			y <- yFromRow(r, 1:nrow(r))
+			w <- abs(cos(pi*y/180))
+			rows <- sample.int(nrow(r), n, replace=TRUE, prob=w)
+			cols <- sample.int(ncol(r), n, replace=TRUE)
+			cells <- cellFromRowCol(r, rows, cols)
 			if (!replace) {
 				cells <- unique(cells)
 			}
-			if (length(cells) > size) {
-				cells <- cells[1:size]
-			} 
 		} else {
-			cells <- sample(ncell(x), size, replace=replace)
+			cells <- sample(ncell(r), size, replace=replace)
 		}
 	} else { # regular 
 		if (lonlat) {
-			ratio <- 0.5 * ncol(x)/nrow(x)
+			ratio <- 0.5 * ncol(r)/nrow(r)
 			n <- sqrt(size)
 			nx <- max(1, (round(n*ratio)))
 			ny <- max(1, (round(n/ratio)))
-			xi <- ncol(x) / nx
-			yi <- nrow(x) / ny
-			rows <- unique(round(seq(.5*yi, nrow(x), yi)))
+			xi <- ncol(r) / nx
+			yi <- nrow(r) / ny
+			rows <- unique(round(seq(.5*yi, nrow(r), yi)))
 
-			w <- cos(pi*yFromRow(x, rows)/180)
+			w <- cos(pi*yFromRow(r, rows)/180)
 			w <- w * length(w)/sum(w)
 			xi <- xi / w
-			xi <- pmax(1,pmin(xi, ncol(x)))
+			xi <- pmax(1,pmin(xi, ncol(r)))
 			z <- list()
 			#off <- stats::runif(1) 			
 			for (i in 1:length(rows)) {
-				z[[i]] <- cbind(rows[i], unique(round(seq(0.5*xi[i], ncol(x), xi[i]))))
+				z[[i]] <- cbind(rows[i], unique(round(seq(0.5*xi[i], ncol(r), xi[i]))))
 			}
 			z <- do.call(rbind, z)
-			cells <- cellFromRowCol(x, z[,1], z[,2])
+			cells <- cellFromRowCol(r, z[,1], z[,2])
 	
 		} else {
-			f <- sqrt(size / ncell(x))
-			nr <- ceiling(nrow(x) * f)
-			nc <- ceiling(ncol(x) * f);
-			xstep <- ncol(x) / nc
-			ystep <- nrow(x) / nr
-			xsamp <- seq(0.5*xstep, ncol(x), xstep)
-			ysamp <- seq(0.5*ystep, nrow(x), ystep)
+			f <- sqrt(size / ncell(r))
+			nr <- ceiling(nrow(r) * f)
+			nc <- ceiling(ncol(r) * f);
+			xstep <- ncol(r) / nc
+			ystep <- nrow(r) / nr
+			xsamp <- seq(0.5*xstep, ncol(r), xstep)
+			ysamp <- seq(0.5*ystep, nrow(r), ystep)
 			xy <- expand.grid(round(ysamp), round(xsamp))
-			cells <- cellFromRowCol(x, xy[,1], xy[,2]) 
+			cells <- cellFromRowCol(r, xy[,1], xy[,2]) 
+		}
+	}
+	if (!is.null(ext)) {
+		cells <- cellFromXY(x, xyFromCell(r, cells))
+	}
+	if (na.rm) {
+		v <- rowSums(is.na(x[cells])) == 0
+		cells <- cells[v]
+	}
+	if (method == "random") {
+		if (length(cells) > n) {
+			cells <- cells[1:n]
 		}
 	}
 	return(cells)
 }
 
 setMethod("spatSample", signature(x="SpatRaster"), 
-	function(x, size, method="regular", replace=FALSE, na.rm=FALSE, as.raster=FALSE, cells=FALSE, ...) {
+	function(x, size, method="random", replace=FALSE, na.rm=FALSE, as.raster=FALSE, cells=FALSE, xy=FALSE, ext=NULL, warn=TRUE) {
 		size <- round(size)
 		if (size < 1) {
 			error("spatSample", "sample size must be a positive integer")
@@ -71,11 +88,14 @@ setMethod("spatSample", signature(x="SpatRaster"),
 			#error("spatSample", "sample size is larger than ncell(x) and replace=FALSE")
 		#}
 		
-		if (cells) {
-			out <- .sampleCells(x, size, method, replace)
+		if (cells || xy) {
+			out <- .sampleCells(x, size, method, replace, na.rm, ext)
 			if (length(out) < size && method == "random") {
 				warn("spatSample", "fewer cells returned than requested")
 			}
+			if (xy) {
+				out <- xyFromCell(x, out)
+			}
 			return(out)
 		}
 		if (!hasValues(x) & !as.raster) {
@@ -88,6 +108,8 @@ setMethod("spatSample", signature(x="SpatRaster"),
 		stopifnot(size > 0)
 		size <- min(ncell(x), size)
 
+		if (!is.null(ext)) x <- crop(x, ext)
+
 		if (method == "regular") {
 			if (as.raster) {
 				x@ptr <- x@ptr$sampleRegularRaster(size)
@@ -144,7 +166,7 @@ setMethod("spatSample", signature(x="SpatRaster"),
 					scells <- .sampleCells(x, size, method, replace)
 					out <- x[scells]
 				}
-				if (NROW(out) < size) {
+				if (warn && (NROW(out) < size)) {
 					warn("spatSample", "fewer values returned than requested")
 				}
 				return(out)
@@ -155,7 +177,7 @@ setMethod("spatSample", signature(x="SpatRaster"),
 
 
 setMethod("spatSample", signature(x="SpatExtent"), 
-	function(x, size, method="regular", lonlat, ...) {
+	function(x, size, method="random", lonlat) {
 		if (missing(lonlat)) {
 			error("spatSample", "provide a lonlat argument")
 		}
@@ -229,9 +251,9 @@ setMethod("spatSample", signature(x="SpatExtent"),
 }
 
 
-coordinates <- function(x) {
-	do.call(cbind, x@ptr$coordinates())
-}
+#coordinates <- function(x) {
+#	do.call(cbind, x@ptr$coordinates())
+#}
 
 get_field_name <- function(x, nms, sender="") {
 	x <- x[1]
@@ -252,23 +274,46 @@ get_field_name <- function(x, nms, sender="") {
 
 
 setMethod("spatSample", signature(x="SpatVector"), 
-	function(x, size, method="regular", by_geom=TRUE, strata=NULL, chess="", ...) {
+	function(x, size, method="random", strata=NULL, chess="") {
 		method = match.arg(tolower(method), c("regular", "random"))
 		stopifnot(size > 0)
 		gtype <- geomtype(x)
 		if (gtype == "polygons") {
-			st <- ""
 			if (!is.null(strata)) {
-				st <- get_field_name(strata, names(x), "spatSample")
-			} 
-			x@ptr = x@ptr$sample(size[1], method[1], by_geom[1], st, .seed())
+				if (length(strata) == 1) {
+					if (is.character(strata)) {
+						stopifnot(strata %in% names(x))
+					} else  {
+						stopifnot((strata > 0) && (strata < ncol(x)))	
+					} 
+					strata <- x[[strata, drop=TRUE]]
+				} else if (length(strata) != length(x)) {
+					stop("length of strata must be 1 or length(x)")
+				}
+				s <- stats::na.omit(unique(strata))
+				n <- length(size)
+				if (n==1) {
+					n <- rep_len(n, length(s)) 
+				} else if (length(s) != n) {
+					stop("length of strata must be 1 or length(na.omit(unique(strata)))")
+				}
+				r <- lapply(s, function(s) {
+					spatSample(x[strata == s, ], size, method, NULL, "")
+				})
+				r <- do.call(c, r)
+				return(r)
+			}
+			if (length(size) == 1) {
+				x@ptr = x@ptr$sample(size, method[1], .seed())
+			} else {
+				x@ptr = x@ptr$sampleGeom(size, method[1], .seed())			
+			}
 			return(messages(x))
 		} else if (grepl(gtype, "points")) {
 			if (!is.null(strata)) {
 				if (inherits(strata, "SpatRaster")) {
-					xy <- coordinates(x)
-					strata <- rast(strata)
-					i <- .grid_sample(xy, size, strata, chess) 
+					xy <- coords(x)
+					i <- .grid_sample(xy, size[1], rast(strata), chess) 
 					return(x[i,])
 				} else {
 					error("spatSample", "not yet implemented for these strata")
