@@ -13,7 +13,7 @@ positive_indices <- function(i, n, caller=" [ ") {
 
 
 setMethod("subset", signature(x="SpatRaster"), 
-function(x, subset, filename="", overwrite=FALSE, wopt=list(), ...) {
+function(x, subset, filename="", overwrite=FALSE, ...) {
 	if (is.character(subset)) {
 		i <- stats::na.omit(match(subset, names(x)))
 		if (length(i)==0) {
@@ -26,7 +26,7 @@ function(x, subset, filename="", overwrite=FALSE, wopt=list(), ...) {
 
 	subset <- as.integer(stats::na.omit(subset) - 1)
 
-	opt <- spatOptions(filename, overwrite, wopt)
+	opt <- spatOptions(filename, overwrite, ...)
 	x@ptr <- x@ptr$subset(subset, opt)
 	messages(x, "subset")
 	return(x)
@@ -64,7 +64,7 @@ function(x, i, j, ... ,drop=TRUE) {
 
 
 setMethod("subset", signature(x="SpatVector"), 
-	function(x, subset, drop=FALSE, ...) {
+	function(x, subset, drop=FALSE) {
 		x <- x[which(as.vector(subset)), , drop=drop]
 		messages(x, "subset")
 	}
@@ -139,7 +139,7 @@ function(x, i, j, ... , drop=FALSE) {
 	x@ptr <- x@ptr$subset_cols(j-1)
 	x <- messages(x, "[")
 	if (drop) {
-		as.data.frame(x, stringsAsFactors=FALSE)
+		as.data.frame(x)
 	} else {
 		x
 	}
