@@ -1,10 +1,10 @@
 
 setMethod("writeStart", signature(x="SpatRaster", filename="character"), 
-	function(x, filename="", overwrite=FALSE, ...) {
-		opt <- spatOptions(filename, overwrite, ...)
+	function(x, filename="", overwrite=FALSE, n=4, ...) {
+		opt <- spatOptions(filename, overwrite, ncopies=n, ...)
 		ok <- x@ptr$writeStart(opt)
 		messages(x, "writeStart")
-		b <- x@ptr$getBlockSize(4, opt$memfrac)
+		b <- x@ptr$getBlockSize(n, opt$memfrac)
 		b$row <- b$row + 1
 		b
 	}
@@ -53,13 +53,14 @@ function(x, filename="", overwrite=FALSE, ...) {
 
 
 setMethod("writeVector", signature(x="SpatVector", filename="character"), 
-function(x, filename, overwrite=FALSE, ...) {
+function(x, filename, filetype="ESRI Shapefile", overwrite=FALSE) {
 	filename <- trimws(filename)
 	if (filename == "") {
 		error("writeVector", "provide a filename")
 	}
-	lyrname <- gsub(".shp", "", basename(filename))
-	success <- x@ptr$write(filename, lyrname, "ESRI Shapefile", overwrite[1])
+	
+	lyrname <- tools::file_path_sans_ext(basename(filename))
+	success <- x@ptr$write(filename, lyrname, filetype, overwrite[1])
 	messages(x, "writeVector")
 	invisible(TRUE)
 }
