@@ -24,11 +24,11 @@ loadModule("spat", TRUE)
 	tv <- utils::packageVersion("terra")
 	#m <- paste("terra version", tv, "(beta-release)")
 	m <- paste("terra version", tv)
-	gdv <- gdal()
-	if (gdv < "3.0.0") {
-		a <- paste("\n\nNOTE: using GDAL version", gdv, "\nFor full functionality you need at least version 3.0.4\n")
-		m <- c(m, a)
-	}
+	#gdv <- gdal()
+	#if (gdv < "3.0.0") {
+	#	a <- paste("\n\nNOTE: using GDAL version", gdv, "\nFor full functionality you need at least version 3.0.4\n")
+	#	m <- c(m, a)
+	#}
 	packageStartupMessage(m)
 	.create_options()
 }
