@@ -1,6 +1,53 @@
 #include "geos_spat.h"
 
 
+std::vector<std::string> SpatVector::wkt() {
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
+	std::vector<std::string> out;
+	out.reserve(g.size());
+	char * wkt;
+	for (size_t i = 0; i < g.size(); i++) {	
+		wkt = GEOSGeomToWKT_r(hGEOSCtxt, g[i].get());
+		out.push_back(wkt);
+	}
+	geos_finish(hGEOSCtxt);
+	return out;
+}
+
+
+std::vector<std::string> SpatVector::wkb() {
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
+	std::vector<std::string> out;
+	out.reserve(g.size());
+	size_t len = 0;
+	for (size_t i = 0; i < g.size(); i++) {	
+		unsigned char *wkb = GEOSGeomToWKB_buf_r(hGEOSCtxt, g[i].get(), &len);
+		std::string s( reinterpret_cast<char const*>(wkb), len) ;
+		out.push_back(s);
+		free(wkb);
+	}
+	geos_finish(hGEOSCtxt);
+	return out;
+}
+
+std::vector<std::string> SpatVector::hex() {
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
+	std::vector<std::string> out;
+	out.reserve(g.size());
+	size_t len = 0;
+	for (size_t i = 0; i < g.size(); i++) {	
+		unsigned char *hex = GEOSGeomToHEX_buf_r(hGEOSCtxt, g[i].get(), &len);
+		std::string s( reinterpret_cast<char const*>(hex), len) ;		
+		out.push_back(s);
+	}
+	geos_finish(hGEOSCtxt);
+	return out;
+}
+
+	
 SpatVector SpatVector::allerretour() {
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
@@ -96,35 +143,30 @@ SpatVector SpatVector::crop(SpatVector v) {
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 
-	std::vector<GeomPtr> x;
+	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	if ((type() != "polygons") & (type() != "mutlipolygons")) {
-		out = convexhull();
-		x = geos_geoms(&out, hGEOSCtxt);
+		v = v.convexhull();
 	} else {
-		x = geos_geoms(this, hGEOSCtxt);
+		v = v.aggregate(false);
 	}
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
 	std::vector<GeomPtr> result;
 	std::vector<unsigned> ids;
-	ids.reserve(size());
 	size_t nx = size();
-	size_t ny = v.size();
-
+	ids.reserve(nx);
 	
 	for (size_t i = 0; i < nx; i++) {
-		for (size_t j = 0; j < ny; j++) {
-			GEOSGeometry* geom = GEOSIntersection_r(hGEOSCtxt, x[i].get(), y[j].get());
-			if (geom == NULL) {
-				out.setError("GEOS exception");
-				geos_finish(hGEOSCtxt);
-				return(out);
-			} 
-			if (!GEOSisEmpty_r(hGEOSCtxt, geom)) {
-				result.push_back(geos_ptr(geom, hGEOSCtxt));
-				ids.push_back(i);
-			} else {
-				GEOSGeom_destroy_r(hGEOSCtxt, geom);
-			}
+		GEOSGeometry* geom = GEOSIntersection_r(hGEOSCtxt, x[i].get(), y[0].get());
+		if (geom == NULL) {
+			out.setError("GEOS exception");
+			geos_finish(hGEOSCtxt);
+			return(out);
+		} 
+		if (!GEOSisEmpty_r(hGEOSCtxt, geom)) {
+			result.push_back(geos_ptr(geom, hGEOSCtxt));
+			ids.push_back(i);
+		} else {
+			GEOSGeom_destroy_r(hGEOSCtxt, geom);
 		}
 	}
 
@@ -295,6 +337,7 @@ SpatVector SpatVector::intersect(SpatVector v) {
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
+	//v = v.aggregate(false);
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
 	std::vector<GeomPtr> result;
 	size_t nx = size();
@@ -302,42 +345,60 @@ SpatVector SpatVector::intersect(SpatVector v) {
 	std::vector<unsigned> idx, idy;
 	idx.reserve(nx);
 	idy.reserve(ny);
-		
-	for (size_t i = 0; i < nx; i++) {
+
+	if (type() == "points") {
+		//std::vector<bool> ixj(nx, false);
+		//size_t count = 0;
 		for (size_t j = 0; j < ny; j++) {
-			GEOSGeometry* geom = GEOSIntersection_r(hGEOSCtxt, x[i].get(), y[j].get());
-			if (geom == NULL) {
-				out.setError("GEOS exception");
-				geos_finish(hGEOSCtxt);
-				return(out);
-			} 
-			if (!GEOSisEmpty_r(hGEOSCtxt, geom)) {
-				result.push_back(geos_ptr(geom, hGEOSCtxt));
-				idx.push_back(i);
-				idy.push_back(j);
-			} else {
-				GEOSGeom_destroy_r(hGEOSCtxt, geom);
+			PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, y[j].get()), hGEOSCtxt);
+			for (size_t i = 0; i < nx; i++) {
+				if (GEOSPreparedIntersects_r(hGEOSCtxt, pr.get(), x[i].get())) {
+					//if (!ixj[i]
+					//ixj[i] = true;
+					idx.push_back(i);
+					idy.push_back(j);
+					//count++;
+				}
 			}
 		}
-	}
-
-	//SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
-
-	if (result.size() > 0) {
-		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
-		out = coll.get(0);
-		out.srs = srs;
-		SpatDataFrame df1 = df.subset_rows(idx);
-		SpatDataFrame df2 = v.df.subset_rows(idy);
-		if (!df1.cbind(df2)) {
-			out.addWarning("could not combine attributes");
+		//std::vector<int> sx;
+		//sx.reserve(count);
+		//for (size_t i=0; i<ixj.size(); i++) {
+		//	if (ixj[i]) sx.push_back(i);	
+		//}
+		out = subset_rows(idx);
+		
+	} else {
+		
+		for (size_t i = 0; i < nx; i++) {
+			for (size_t j = 0; j < ny; j++) {
+				GEOSGeometry* geom = GEOSIntersection_r(hGEOSCtxt, x[i].get(), y[j].get());
+				if (geom == NULL) {
+					out.setError("GEOS exception");
+					geos_finish(hGEOSCtxt);
+					return(out);
+				} 
+				if (!GEOSisEmpty_r(hGEOSCtxt, geom)) {
+					result.push_back(geos_ptr(geom, hGEOSCtxt));
+					idx.push_back(i);
+					idy.push_back(j);
+				} else {
+					GEOSGeom_destroy_r(hGEOSCtxt, geom);
+				}
+			}
 		}
-		out.df = df1;
-	} 
+	//SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
+		if (result.size() > 0) {
+			SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
+			out = coll.get(0);
+			out.srs = srs;
+		}
+	}
+	geos_finish(hGEOSCtxt);
+
 	if (!srs.is_same(v.srs, true)) {
 		out.addWarning("different crs"); 
 	}
-	geos_finish(hGEOSCtxt);
 
 	if ((type() == "polygons") && (v.type() == "polygons") && (out.type() != "polygons")) {
 		// intersection is point or line, return empty 
@@ -346,11 +407,17 @@ SpatVector SpatVector::intersect(SpatVector v) {
 		out.srs = srs;
 	}
 
+	SpatDataFrame df1 = df.subset_rows(idx);
+	SpatDataFrame df2 = v.df.subset_rows(idy);
+	if (!df1.cbind(df2)) {
+		out.addWarning("could not combine attributes");
+	}
+	out.df = df1;
 	return out;
 }
 
 
-
+/*
 
 std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> getRelateFun(const std::string rel) {
 	std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> rfun;
@@ -366,6 +433,8 @@ std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry
 		rfun = GEOSWithin_r;
 	} else if (rel == "contains") {
 		rfun = GEOSContains_r;
+	} else if (rel == "containsproperly") {
+		rfun = GEOSContainsProperly_r;
 	} else if (rel == "overlaps") {
 		rfun = GEOSOverlaps_r;
 	} else if (rel == "covers") {
@@ -375,7 +444,33 @@ std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry
 	}
 	return rfun;
 }
+*/
 
+std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> getPrepRelateFun(const std::string rel) {
+	std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> rfun;
+	if (rel == "intersects") {
+		rfun = GEOSPreparedIntersects_r;
+	} else if (rel == "disjoint") {
+		rfun = GEOSPreparedDisjoint_r;
+	} else if (rel == "touches") {
+		rfun = GEOSPreparedTouches_r;
+	} else if (rel == "crosses") {
+		rfun = GEOSPreparedCrosses_r;
+	} else if (rel == "within") {
+		rfun = GEOSPreparedWithin_r;
+	} else if (rel == "contains") {
+		rfun = GEOSPreparedContains_r;
+	} else if (rel == "containsproperly") {
+		rfun = GEOSPreparedContainsProperly_r;
+	} else if (rel == "overlaps") {
+		rfun = GEOSPreparedOverlaps_r;
+	} else if (rel == "covers") {
+		rfun = GEOSPreparedCovers_r;
+	} else if (rel == "coveredby") {
+		rfun = GEOSPreparedCoveredBy_r;
+	}
+	return rfun;
+}
 
 
 int getRel(std::string &relation) {
@@ -414,6 +509,7 @@ std::vector<int> SpatVector::relate(SpatVector v, std::string relation) {
 		setError("'" + relation + "'" + " is not a valid relate name or pattern");
 		return out;
 	}
+
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
@@ -427,14 +523,16 @@ std::vector<int> SpatVector::relate(SpatVector v, std::string relation) {
 			}
 		}
 	} else {
-		std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+		std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> relFun = getPrepRelateFun(relation);
 		for (size_t i = 0; i < nx; i++) {
+			PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, x[i].get()), hGEOSCtxt);
 			for (size_t j = 0; j < ny; j++) {
-				out.push_back( relFun(hGEOSCtxt, x[i].get(), y[j].get()));
+				out.push_back( relFun(hGEOSCtxt, pr.get(), y[j].get()));
 			}
 		} 
 	}
 	geos_finish(hGEOSCtxt);
+
 	return out;
 }
 
@@ -463,16 +561,18 @@ std::vector<int> SpatVector::relateFirst(SpatVector v, std::string relation) {
 			}
 		}
 	} else {
-		std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+		//std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+		std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> relFun = getPrepRelateFun(relation);
 
 		for (size_t i = 0; i < nx; i++) {
+			PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, x[i].get()), hGEOSCtxt);
 			for (size_t j = 0; j < ny; j++) {
-				if (relFun(hGEOSCtxt, x[i].get(), y[j].get())) {
+				if (relFun(hGEOSCtxt, pr.get(), y[j].get())) {
 					out[i] = j;
-					continue;					
+					continue;
 				}
 			}
-		} 
+		}
 	}
 	geos_finish(hGEOSCtxt);
 	return out;
@@ -491,7 +591,7 @@ std::vector<int> SpatVector::relate(std::string relation, bool symmetrical) {
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
-	
+
 	if (symmetrical) {
 		size_t s = size();
 		size_t n = ((s-1) * s)/2;
@@ -503,10 +603,11 @@ std::vector<int> SpatVector::relate(std::string relation, bool symmetrical) {
 				}
 			}
 		} else {
-			std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+			std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> relFun = getPrepRelateFun(relation);
 			for (size_t i=0; i<(s-1); i++) {
+				PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, x[i].get()), hGEOSCtxt);
 				for (size_t j=(i+1); j<s; j++) {
-					out.push_back( relFun(hGEOSCtxt, x[i].get(), x[j].get()));
+					out.push_back( relFun(hGEOSCtxt, pr.get(), x[j].get()));
 				}
 			} 			
 		}
@@ -520,15 +621,18 @@ std::vector<int> SpatVector::relate(std::string relation, bool symmetrical) {
 				}
 			}
 		} else {
-			std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+			std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> relFun = getPrepRelateFun(relation);
 			for (size_t i = 0; i < nx; i++) {
+				PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, x[i].get()), hGEOSCtxt);
 				for (size_t j = 0; j < nx; j++) {
-					out.push_back( relFun(hGEOSCtxt, x[i].get(), x[j].get()));
+					out.push_back( relFun(hGEOSCtxt, pr.get(), x[j].get()));
 				}
 			} 
 		}
 	}
+
 	geos_finish(hGEOSCtxt);
+
 	return out;
 }
 
@@ -544,7 +648,7 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel) {
 	size_t nx = size();
 	size_t ny = v.size();
 	double d;
-	
+
 	if (parallel) {
 		bool nyone = false;
 		if (nx != ny) {
@@ -825,10 +929,10 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 		std::vector<GeomPtr> b(size());
 		for (size_t i=0; i < x.size(); i++) {	
 			GEOSCoordSequence* csq = GEOSNearestPoints_r(hGEOSCtxt, x[i].get(), y[i].get());
-			GEOSGeometry* geom = GEOSGeom_createPoint_r(hGEOSCtxt, csq);
+			GEOSGeometry* geom = GEOSGeom_createLineString_r(hGEOSCtxt, csq);
 			b[i] = geos_ptr(geom, hGEOSCtxt);
 		}
-		out = vect_from_geos(b, hGEOSCtxt, "points");
+		out = vect_from_geos(b, hGEOSCtxt, "lines");
 		
 	} else {	
 		SpatVector mp = v.aggregate(false);
@@ -837,12 +941,13 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 		std::vector<GeomPtr> b(size());
 		for (size_t i = 0; i < x.size(); i++) {	
 			GEOSCoordSequence* csq = GEOSNearestPoints_r(hGEOSCtxt, x[i].get(), y[0].get());
-			GEOSGeometry* geom = GEOSGeom_createPoint_r(hGEOSCtxt, csq);
+			GEOSGeometry* geom = GEOSGeom_createLineString_r(hGEOSCtxt, csq);
 			b[i] = geos_ptr(geom, hGEOSCtxt);
 		}
-		out = vect_from_geos(b, hGEOSCtxt, "points");
+		out = vect_from_geos(b, hGEOSCtxt, "lines");
 	}
 	geos_finish(hGEOSCtxt);
+	out.srs = srs;
 	return out;
 }
 
@@ -856,17 +961,19 @@ SpatVector SpatVector::nearest_point() {
 		out.addWarning("single geometry");
 		//return *this;
 	}
+	size_t n = size();
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
-	SpatVector xa = aggregate(false);
-	std::vector<GeomPtr> y = geos_geoms(&xa, hGEOSCtxt);
-	std::vector<GeomPtr> b(size());
-	for (size_t i = 0; i < x.size(); i++) {	
+	std::vector<GeomPtr> b(n);
+	for (unsigned i = 0; i < n; i++) {	
+		SpatVector xa = remove_rows({i});
+		xa = xa.aggregate(false);
+		std::vector<GeomPtr> y = geos_geoms(&xa, hGEOSCtxt);
 		GEOSCoordSequence* csq = GEOSNearestPoints_r(hGEOSCtxt, x[i].get(), y[0].get());
-		GEOSGeometry* geom = GEOSGeom_createPoint_r(hGEOSCtxt, csq);
+		GEOSGeometry* geom = GEOSGeom_createLineString_r(hGEOSCtxt, csq);
 		b[i] = geos_ptr(geom, hGEOSCtxt);
 	}
-	out = vect_from_geos(b, hGEOSCtxt, "points");
+	out = vect_from_geos(b, hGEOSCtxt, "lines");
 	geos_finish(hGEOSCtxt);
 	out.srs = srs;
 	return out;
