@@ -7,9 +7,12 @@ std::vector<char *> string_to_charpnt(std::vector<std::string> s);
 
 std::string concatenate(std::vector<std::string> v, std::string delim);
 void lowercase(std::string &s);
+void lowercase(std::vector<std::string> &ss);
+
 std::string is_in_set_default(std::string s, std::vector<std::string> ss, std::string defvalue, bool tolower);
+int where_in_set(std::string s, std::vector<std::string> ss, bool tolower);
 bool is_in_vector(std::string s, std::vector<std::string> ss);
-int where_in_vector(std::string s, std::vector<std::string> ss);
+int where_in_vector(std::string s, std::vector<std::string> ss, bool tolower);
 std::vector<std::string> getlastpart (std::vector<std::string> s, std::string delim);
 
 std::vector<std::string> strsplit(std::string s, std::string delimiter);
