@@ -95,7 +95,7 @@ SpatVector SpatVector::disaggregate() {
 SpatVector SpatVector::aggregate(std::string field, bool dissolve) {
 
 	SpatVector out;
-	int i = where_in_vector(field, get_names());
+	int i = where_in_vector(field, get_names(), false);
 	if (i < 0) {
 		out.setError("cannot find field");
 		return out;	
